(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["module", "exports"], factory);
  } else if (typeof exports !== "undefined") {
    factory(module, exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod, mod.exports);
    global.undefined = mod.exports;
  }
})(this, function (module, exports) {
  "use strict";

  (function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
      var v = factory(require, exports);
      if (v !== undefined) module.exports = v;
    } else if (typeof define === "function" && define.amd) {
      define(["require", "exports", "yargs"], factory);
    }
  })(function (require, exports) {
    "use strict";

    Object.defineProperty(exports, "__esModule", {
      value: true
    });

    const yargs = require("yargs");

    var BuildErrorLevel;

    (function (BuildErrorLevel) {
      BuildErrorLevel[BuildErrorLevel["Off"] = 0] = "Off";
      BuildErrorLevel[BuildErrorLevel["Warning"] = 1] = "Warning";
      BuildErrorLevel[BuildErrorLevel["Error"] = 2] = "Error";
    })(BuildErrorLevel = exports.BuildErrorLevel || (exports.BuildErrorLevel = {})); // Command line option:
    //  --fatal=[warning|error|off]


    const fatalLevelArgument = yargs.argv.fatal;

    class BuildErrorHandler {
      static isFatal(level) {
        let fatalLevel;

        switch (fatalLevelArgument) {
          case "error":
            fatalLevel = BuildErrorLevel.Error;
            break;

          case "warning":
            fatalLevel = BuildErrorLevel.Warning;
            break;

          case "off":
            return false;

          default:
            fatalLevel = BuildErrorLevel.Error;
            break;
        }

        return fatalLevel <= level;
      }

      static setCustomHandler(handler) {
        BuildErrorHandler.handleError = handler;
      }

      static handleError(level, error) {
        if (BuildErrorHandler.isFatal(level)) {
          process.exit(1);
        } else {
          console.log(error.message);
        }
      }

      static onError(error) {
        BuildErrorHandler.handleError(BuildErrorLevel.Error, error);
      }

      static onWarning(error) {
        BuildErrorHandler.handleError(BuildErrorLevel.Warning, error);
      }

    }

    exports.default = BuildErrorHandler;
  });
});
//# sourceMappingURL=build-error-handler.js.map
