(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["module", "exports"], factory);
  } else if (typeof exports !== "undefined") {
    factory(module, exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod, mod.exports);
    global.undefined = mod.exports;
  }
})(this, function (module, exports) {
  "use strict";

  (function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
      var v = factory(require, exports);
      if (v !== undefined) module.exports = v;
    } else if (typeof define === "function" && define.amd) {
      define(["require", "exports", "child_process", "./stream-cache"], factory);
    }
  })(function (require, exports) {
    "use strict";

    Object.defineProperty(exports, "__esModule", {
      value: true
    });

    const child_process_1 = require("child_process");

    const stream_cache_1 = require("./stream-cache");

    class ChildProcess {
      static spawnAsync(command, args, options) {
        let bufferOut;
        let bufferErr;
        if (!args) args = [];
        let spawnCommandAndArgs = command + " " + args.join(" ");
        if (options && options.echo) console.log(spawnCommandAndArgs);

        if (options && options.silentUntilError) {
          bufferOut = new stream_cache_1.default();
          bufferErr = new stream_cache_1.default();
        }

        return new Promise((resolve, reject) => {
          let failed = false;
          let proc = child_process_1.spawn(command, args, options);

          if (bufferOut) {
            proc.stdout.pipe(bufferOut);
            proc.stderr.pipe(bufferErr);
          }

          proc.once("error", error => {
            failed = true;

            if (bufferOut) {
              bufferOut.pipe(process.stdout, {
                end: false
              });
              bufferOut.end();
              bufferOut = null;
              bufferErr.pipe(process.stderr, {
                end: false
              });
              bufferErr.end();
              bufferErr = null;
            }

            reject(error);
          });
          proc.once("exit", code => {
            if (code === 0) resolve(proc);else {
              if (!failed) {
                let error = new Error("Exit code = " + code + " for:\n" + spawnCommandAndArgs + "\n");
                error.childProcess = proc;

                if (bufferOut) {
                  bufferOut.pipe(process.stdout, {
                    end: false
                  });
                  bufferOut.end();
                  bufferOut = null;
                  bufferErr.pipe(process.stderr, {
                    end: false
                  });
                  bufferErr.end();
                  bufferErr = null;
                }

                reject(error);
              }
            }
          });
        });
      }

    }

    exports.default = ChildProcess;
  });
});
//# sourceMappingURL=child-process.js.map
