(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["module", "exports"], factory);
  } else if (typeof exports !== "undefined") {
    factory(module, exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod, mod.exports);
    global.undefined = mod.exports;
  }
})(this, function (module, exports) {
  "use strict";

  (function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
      var v = factory(require, exports);
      if (v !== undefined) module.exports = v;
    } else if (typeof define === "function" && define.amd) {
      define(["require", "exports", "gulp", "path"], factory);
    }
  })(function (require, exports) {
    "use strict";

    Object.defineProperty(exports, "__esModule", {
      value: true
    });

    const gulp = require("gulp");

    const path = require("path");

    class DotNetModuleReference {
      constructor(projectPath, nodeModule) {
        this._projectPath = projectPath;
        this._nodeModule = nodeModule;
      }

      get projectPath() {
        return this._projectPath;
      }

      get nodeModule() {
        return this._nodeModule;
      }

      getAssemblySourceDir(assemblyName) {
        return path.join(this._nodeModule.packageDir, "Source", assemblyName);
      }

      getAssemblyBinaryDir(assemblyName) {
        const sourceDir = this.getAssemblySourceDir(assemblyName);
        return path.join(sourceDir, "bin", "Release");
      }

      async importAssemblyAsync(packageName, assemblyName) {
        const binDestDir = path.join(this._projectPath, "Source", "packages", packageName, "lib");
        let binAssemblies = [];
        binAssemblies = binAssemblies.concat(assemblyName);
        let promises = [];
        binAssemblies.forEach(assembly => {
          const srcDir = this.getAssemblyBinaryDir(assembly);
          promises = promises.concat(this.copyPatternsAsync(path.join(srcDir, assembly + ".dll"), binDestDir, {
            base: srcDir
          }));
        });
        await Promise.all(promises);
      }

      async copyPatternsAsync(sourcePatterns, destination, options) {
        await new Promise((resolve, reject) => {
          let destinationOptions;
          gulp.src(sourcePatterns, options).pipe(gulp.dest(destination, destinationOptions)).on("end", resolve).on("error", reject);
        });
      }

    }

    exports.default = DotNetModuleReference;
  });
});
//# sourceMappingURL=dotnet-module-reference.js.map
