(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["module", "exports"], factory);
  } else if (typeof exports !== "undefined") {
    factory(module, exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod, mod.exports);
    global.undefined = mod.exports;
  }
})(this, function (module, exports) {
  "use strict";

  (function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
      var v = factory(require, exports);
      if (v !== undefined) module.exports = v;
    } else if (typeof define === "function" && define.amd) {
      define(["require", "exports", "bluebird", "gulp", "gulp-flatten", "path", "./child-process", "fs-extra"], factory);
    }
  })(function (require, exports) {
    "use strict";

    Object.defineProperty(exports, "__esModule", {
      value: true
    });

    const Bluebird = require("bluebird");

    const gulp = require("gulp");

    const flatten = require("gulp-flatten");

    const path = require("path");

    const child_process_1 = require("./child-process");

    const fs = require("fs-extra");

    let fsp = Bluebird.promisifyAll(fs);

    class MonoDocBuilder {
      static getAssemblyDirOrDefault(options) {
        if (options && options.assemblyDir) return options.assemblyDir;
        return path.join("Source", "${assembly}", "bin", "Release");
      }

      static getOutDirOrDefault(options) {
        if (options && options.outDir) return options.outDir;
        return "MonoDoc";
      }

      static getAssembleDirNameOrDefault(options) {
        if (options && options.assembleDirName) return options.assembleDirName;
        return "assemble";
      }

      static getXmlDirNameOrDefault(options) {
        if (options && options.xmlDirName) return options.xmlDirName;
        return "xml";
      }

      get assembleDir() {
        return path.join(this.outDir, this.assembleDirName);
      }

      get xmlDir() {
        return path.join(this.outDir, this.xmlDirName);
      }

      constructor(options) {
        this.assemblyDir = MonoDocBuilder.getAssemblyDirOrDefault(options);
        this.outDir = MonoDocBuilder.getOutDirOrDefault(options);
        this.assembleDirName = MonoDocBuilder.getAssembleDirNameOrDefault(options);
        this.xmlDirName = MonoDocBuilder.getXmlDirNameOrDefault(options);
      }

      async updateAsync(assemblies, referencePaths) {
        // Drop .dll
        for (let i = 0; i < assemblies.length; ++i) {
          assemblies[i] = assemblies[i].replace(".dll", "");
        }

        let assemblyPaths = [];
        assemblies.forEach(a => {
          assemblyPaths = assemblyPaths.concat(path.join(this.assemblyDir.replace("${assembly}", a), a + ".dll"));
        });
        let xmlParams = [];
        assemblies.forEach(a => {
          xmlParams = xmlParams.concat("-i");
          xmlParams = xmlParams.concat(path.join(this.assemblyDir.replace("${assembly}", a), a + ".xml"));
        });
        let referenceParams = [];

        if (referencePaths) {
          referencePaths.forEach(r => {
            referenceParams = referenceParams.concat("-L");
            referenceParams = referenceParams.concat(r);
          });
        }

        let params = ["update", "-out:" + this.xmlDir].concat(xmlParams).concat(referenceParams).concat(assemblyPaths);
        await child_process_1.default.spawnAsync("mdoc", params, {
          stdio: "inherit"
        });
      }

      async assembleAsync(prefix) {
        console.log("Assembling MonoDocs ...");
        await fsp.mkdirpAsync(this.assembleDir);
        let prefixPath = path.join(this.assembleDir, prefix);
        await child_process_1.default.spawnAsync("mdoc", ["assemble", "-o", prefixPath, this.xmlDir], {
          stdio: "inherit"
        });
      }

      async installAsync() {
        await new Promise((resolve, reject) => {
          this.getInstallPathAsync().then(installPath => {
            gulp.src([path.join(this.assembleDir, "*"), path.join(this.outDir, "*.source")]).pipe(flatten()).pipe(gulp.dest(installPath)).on("end", resolve).on("error", reject);
          });
        });
      }

      async buildAsync(prefix, assemblies, referencePaths) {
        await this.updateAsync(assemblies, referencePaths);
        await this.assembleAsync(prefix);
      }

      async getInstallPathAsync() {
        return Promise.resolve("/Library/Frameworks/Mono.framework/External/monodoc/"); // execute("pkg-config monodoc --variable=sourcesdir", callback);
      }

    }

    exports.default = MonoDocBuilder;
  });
});
//# sourceMappingURL=monodoc-builder.js.map
