(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["module", "exports"], factory);
  } else if (typeof exports !== "undefined") {
    factory(module, exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod, mod.exports);
    global.undefined = mod.exports;
  }
})(this, function (module, exports) {
  "use strict";

  (function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
      var v = factory(require, exports);
      if (v !== undefined) module.exports = v;
    } else if (typeof define === "function" && define.amd) {
      define(["require", "exports", "gulp", "gulp-msbuild", "./child-process"], factory);
    }
  })(function (require, exports) {
    "use strict";

    Object.defineProperty(exports, "__esModule", {
      value: true
    });

    const gulp = require("gulp");

    const msbuild = require("gulp-msbuild");

    const child_process_1 = require("./child-process");

    class MSBuildBuilder {
      static async buildAsync(sourcePatternsOrOptions, options) {
        let sourcePatterns;

        if (typeof sourcePatternsOrOptions === "string" || Array.isArray(sourcePatternsOrOptions)) {
          sourcePatterns = sourcePatternsOrOptions;
        } else {
          options = sourcePatternsOrOptions;
        }

        if (!options) options = {};
        if (options.errorOnFail === undefined) options.errorOnFail = true;
        if (options.emitEndEvent === undefined) options.emitEndEvent = true;
        if (options.stdout === undefined) options.stdout = true;

        if (process.platform.match(/darwin/)) {
          options.msbuildPath = "msbuild";
          if (!options.toolsVersion) options.toolsVersion = undefined;
        }

        if (!sourcePatterns) {
          sourcePatterns = "**/*.sln";
          if (!options.targets) options.targets = ["Build"];
        }

        await new Promise((resolve, reject) => {
          gulp.src(sourcePatterns).pipe(msbuild(options)).once("end", resolve).once("error", reject);
        });
      }

      static async restoreAsync(sourcePatterns) {
        if (!sourcePatterns || sourcePatterns.length === 0) {
          sourcePatterns = "**/*.sln";
        }

        if (process.platform.match(/darwin/)) {
          await child_process_1.default.spawnAsync("nuget", ["restore"].concat(sourcePatterns), {
            stdio: "inherit"
          });
        } else {
          let options = {};
          options.targets = ["restore"];
          await this.buildAsync(sourcePatterns, options);
        }
      }

    }

    exports.default = MSBuildBuilder;
  });
});
//# sourceMappingURL=msbuild-builder.js.map
