(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["module", "exports"], factory);
  } else if (typeof exports !== "undefined") {
    factory(module, exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod, mod.exports);
    global.undefined = mod.exports;
  }
})(this, function (module, exports) {
  "use strict";

  /// <reference types="node" />
  (function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
      var v = factory(require, exports);
      if (v !== undefined) module.exports = v;
    } else if (typeof define === "function" && define.amd) {
      define(["require", "exports", "stream"], factory);
    }
  })(function (require, exports) {
    "use strict";

    Object.defineProperty(exports, "__esModule", {
      value: true
    });

    const stream = require("stream");

    class StreamCache extends stream.Stream {
      constructor() {
        super();
        this.writable = true;
        this.readable = true;
        this._buffers = [];
        this._dests = [];
        this._ended = false;
      }

      write(buffer, encoding, cb) {
        this._buffers.push(buffer);

        this._dests.forEach(function (dest) {
          dest.write(buffer);
        });

        return true;
      }

      pipe(dest, options) {
        this._buffers.forEach(function (buffer) {
          dest.write(buffer);
        });

        if (this._ended) {
          if (options && options.end) dest.end();
          return dest;
        }

        this._dests.push(dest);

        return dest;
      }

      getLength() {
        return this._buffers.reduce(function (totalLength, buffer) {
          return totalLength + buffer.length;
        }, 0);
      }

      end() {
        this._dests.forEach(function (dest) {
          dest.end();
        });

        this._ended = true;
        this._dests = [];
      }

    }

    exports.default = StreamCache;
  });
});
//# sourceMappingURL=stream-cache.js.map
