(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports", "gulp", "gulp-util", "gulp-if", "gulp-typescript", "gulp-tslint", "typescript", "gulp-babel", "event-stream", "gulp-sourcemaps", "path"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require("gulp"), require("gulp-util"), require("gulp-if"), require("gulp-typescript"), require("gulp-tslint"), require("typescript"), require("gulp-babel"), require("event-stream"), require("gulp-sourcemaps"), require("path"));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.gulp, global.gulpUtil, global.gulpIf, global.gulpTypescript, global.gulpTslint, global.typescript, global.gulpBabel, global.eventStream, global.gulpSourcemaps, global.path);
    global.undefined = mod.exports;
  }
})(this, function (exports, _gulp, _gulpUtil, _gulpIf, _gulpTypescript, _gulpTslint, _typescript, _gulpBabel, _eventStream, _gulpSourcemaps, _path) {
  "use strict"; // import Promise from "bluebird";
  // Tools

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _gulp2 = _interopRequireDefault(_gulp);

  var _gulpUtil2 = _interopRequireDefault(_gulpUtil);

  var _gulpIf2 = _interopRequireDefault(_gulpIf);

  var _gulpTypescript2 = _interopRequireDefault(_gulpTypescript);

  var _gulpTslint2 = _interopRequireDefault(_gulpTslint);

  var _typescript2 = _interopRequireDefault(_typescript);

  var _gulpBabel2 = _interopRequireDefault(_gulpBabel);

  var _eventStream2 = _interopRequireDefault(_eventStream);

  var _gulpSourcemaps2 = _interopRequireDefault(_gulpSourcemaps);

  var _path2 = _interopRequireDefault(_path);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  // Streams and process
  class TypeSriptBuilder {
    static async buildAsync(projectOrOptions, options) {
      let project = projectOrOptions;

      if (projectOrOptions && projectOrOptions.typescript) {
        // project
        if (!options) options = {};
      } else {
        if (!options) options = projectOrOptions;
        if (!options) options = {};
        if (!options.projectFile) options.projectFile = "tsconfig.json";
        if (!options.compilerOptions) options.compilerOptions = {};
        if (!options.compilerOptions.typescript) options.compilerOptions.typescript = _typescript2.default;
        project = _gulpTypescript2.default.createProject(options.projectFile, options.compilerOptions);
      }

      if (options.lint === undefined) options.lint = true;

      if (options.babel === undefined) {
        // Node doesn't fully support anything beyond ES5 and will require babel
        if (project.options.target > _typescript2.default.ScriptTarget.ES5) options.babel = true;else options.babel = false;
      }

      let rootDir = project.options.rootDir || "./";
      let outDir = project.options.outDir || "./";

      let mapRoot = project.options.mapRoot || _path2.default.relative(outDir, "./");

      let sourceRoot = project.options.sourceRoot || _path2.default.relative(outDir, rootDir);

      let tsSources = _path2.default.join(rootDir, "**/*.ts"); // includes ambient


      let dtsSources = _path2.default.join(rootDir, "**/*.d.ts"); // ambient only


      let jsSources = _path2.default.join(rootDir, "**/*.js");

      _gulpUtil2.default.log("Starting '" + _gulpUtil2.default.colors.cyan("TypeScript lint") + "'...");

      await new Promise((resolve, reject) => {
        let lint;

        if (options.lint) {
          _gulpUtil2.default.log("Linting ...");

          lint = _gulp2.default.src(tsSources).pipe((0, _gulpTslint2.default)()).pipe(_gulpTslint2.default.report());
        } else {
          _gulpUtil2.default.log("Skip linting ...");

          lint = _eventStream2.default.merge([]);
        }

        lint.once("end", () => {
          _gulpUtil2.default.log("Finished '" + _gulpUtil2.default.colors.cyan("TypeScript lint") + "'");

          resolve();
        }).once("error", reject);
      });

      _gulpUtil2.default.log("Starting '" + _gulpUtil2.default.colors.cyan("TypeScript compile") + "'...");

      await new Promise((resolve, reject) => {
        let tsc = project.src().pipe((0, _gulpIf2.default)(project.options.sourceMap, _gulpSourcemaps2.default.init({
          loadMaps: true
        }))).pipe(project()).once("end", () => _gulpUtil2.default.log("Processed TypeScript project")).once("error", reject);
        let js = tsc.js.pipe((0, _gulpIf2.default)(options.babel, (0, _gulpBabel2.default)())).pipe((0, _gulpIf2.default)(project.options.sourceMap, _gulpSourcemaps2.default.write(mapRoot, {
          includeContent: false,
          sourceRoot: sourceRoot
        }))).pipe(_gulp2.default.dest(outDir)).once("end", () => _gulpUtil2.default.log("Processed js"));
        let dts = tsc.dts.pipe(_gulp2.default.dest(outDir)).once("end", () => _gulpUtil2.default.log("Processed dts"));

        _eventStream2.default.merge(js, dts).once("end", () => {
          _gulpUtil2.default.log("Finished '" + _gulpUtil2.default.colors.cyan("TypeScript compile") + "'");

          resolve();
        }).once("error", reject);
      });

      _gulpUtil2.default.log("Starting '" + _gulpUtil2.default.colors.cyan("Copy dts") + "'...");

      await new Promise((resolve, reject) => {
        // Copy extra ambient declarations
        _gulp2.default.src(dtsSources, {
          base: rootDir
        }).pipe(_gulp2.default.dest(outDir)).once("end", () => {
          _gulpUtil2.default.log("Finished '" + _gulpUtil2.default.colors.cyan("Copy dts") + "'");

          resolve();
        }).once("error", reject);
      });

      _gulpUtil2.default.log("Starting '" + _gulpUtil2.default.colors.cyan("Copy js") + "'...");

      await new Promise((resolve, reject) => {
        // Copy extra JS
        _gulp2.default.src(jsSources, {
          base: rootDir
        }).pipe((0, _gulpIf2.default)(options.babel, (0, _gulpBabel2.default)())).pipe(_gulp2.default.dest(outDir)).once("end", () => {
          _gulpUtil2.default.log("Finished '" + _gulpUtil2.default.colors.cyan("Copy js") + "'");

          resolve();
        }).once("error", reject);
      });
    }

  }

  exports.default = TypeSriptBuilder;
});